/*
hardware.h

Copyright 1998-2001 Larry Barello

Author: Larry Barello
        larry@barello.net

Defines all CPU peripheral definitions/IO assignments, etc.
*/

// Here is the CPU stuff

#define CPUCLK      8000000
#define BAUDRATE 19200
#define UBRR_INIT ((CPUCLK/16/BAUDRATE)-1)
//
// System Clock settings: CPU/64, FREE RUN ROLLOVER!
// The R/C Pulse measurement relies on the 8 bit timer
// being free running.  So, specify the tickrate in terms
// of the CPU clock and DON'T use the reload value!
//
#define TICKRATE     (CPUCLK/64/256)
#define MILLISECOND(A) ((A)*(long)TICKRATE/1000)
#define HERTZ(A)    (TICKRATE/(A))
#define TCCR0_INIT  (1<<CS01 | 1<<CS00) // CPUCLK/64
//
// MCUCR = Int0/1 posative edge triggered, Sleep enabled
// TIMSK = Enable timer0 overflow interrupt
// GIMSK = Enable int0/1 interrupts
// ACSR  = Enable analog comparator, posative edge interrupt.
//
#define MCUCR_INIT ((1<<SE) | (1<<ISC11) | (1<<ISC10) | (1<<ISC00) | (1<<ISC01))
#define TIMSK_INIT (1<<TOIE0)   // No Timer1 overflow...
//#define TIMSK_INIT (1<<TOIE0 | 1<<TOIE1)
#define GIMSK_INIT ((1<<INT1) | (1<<INT0))
#define ACSR_INIT ((1<<ACIE) | (1<<ACIS1) | (1<<ACIS0))
//
// High Speed Counter settings:  CPU/1,8 and 64, 8, 9 and 10 bit PWM 
//
#define TCCR1A_8BIT  ((1<<COM1A1) | (1<<COM1B1) | (1<<PWM10))
#define TCCR1A_9BIT  ((1<<COM1A1) | (1<<COM1B1) | (1<<PWM11))
#define TCCR1A_10BIT ((1<<COM1A1) | (1<<COM1B1) | (1<<PWM10) | (1<<PWM11))

#define TCCR1B_CLK1D (1<<CS10 | 1<<CTC1)    /* CLK * 2              */
#define TCCR1B_CLK1  (1<<CS10)              /* CLK                  */
#define TCCR1B_CLK8D (1<<CS11 | 1<<CTC1)    /* CLK/4 (CLK * 2 / 8)  */
#define TCCR1B_CLK8  (1<<CS11)              /* CLK/8                */
//
// Right Motor
// PD5 = OC1A
//
#define RIGHT_DRIVEH OCR1AH
#define RIGHT_DRIVEL OCR1AL
//
// Left Motor
// PD4 (OC1B)
//
#define LEFT_DRIVEH  OCR1BH
#define LEFT_DRIVEL  OCR1BL

// I/O Pin assignments and initialization bytes

/* PORTA

    0 - in  unassigned
    1 - in  unassigned
    2 - in  unassigned
    3 - in  unassigned
    4 - in  Right Battery
    5 - in  Left Battery
    6 - out AUX LOW  Default 1 (off)
    7 - out AUX HIGH Default 1 (off)

*/
#define DDRA_INIT 0xC0
#define PORTA_INIT 0xC0

#define ADCLeftBattery  PA5		// OSMC Supply voltage sense
#define ADCRightBattery PA4
#define VFULLSCALE 50           // Full scale voltage

// Auxillary OPTO outputs

#define AUXLOW_ON() cbi(PORTA, PA6)
#define AUXLOW_OFF() sbi(PORTA, PA6)
#define AUXHIGH_ON() cbi(PORTA, PA7)
#define AUXHIGH_OFF() sbi(PORTA, PA7)

/* PORTB

    0 - out Right-Dir Default 0
    1 - out Left-Dir  Default 0
    2 - in  Aux input
    3 - in  Threshold for Aux input (no pullup!)
    4 - NA  LCD D4
    5 - NA  LCD D5
    6 - NA  LCD D6
    7 - NA  LCD D7
*/
#define DDRB_INIT 0x03
#define PORTB_INIT 0xF7  // Pull up undefined inputs

#define LEFT_FORWARD()  sbi(PORTB, PB1)
#define LEFT_REVERSE()  cbi(PORTB, PB1)
#define RIGHT_FORWARD() sbi(PORTB, PB0)
#define RIGHT_REVERSE() cbi(PORTB, PB0)

// Need to lower B_EN line to read switches.  Mutually exclusive
// with the LCD data buss

#define LCD_SW0() (inp(PINB) & BV(PB4))
#define LCD_SW1() (inp(PINB) & BV(PB5))
#define LCD_SW2() (inp(PINB) & BV(PB6))
#define LCD_SW3() (inp(PINB) & BV(PB7))

#define LCD_SWITCHES() (inp(PINB) & 0xF0)

#define SW_UP 4
#define SW_LEFT 5
#define SW_RIGHT 6
#define SW_DOWN 7

/* PORTC

    0 - NA  SCL  or Flip input
    1 - NA  SDA
    2 - out PWM Mode Select default low
    3 - out RIGHT DISABLE Default high
    4 - out LCD/Switch select (default 1)
    5 - out LCD_RS
    6 - out LCD_RW
    7 - out LCD_E
*/
#define DDRC_INIT 0xFC	// 1111 1100
#define PORTC_INIT 0x1F // 0001 1111

#define FLIP_IN() (inp(PINC) & BV(PC0))

#define RIGHT_DISABLE() sbi(PORTC, PC3)
#define RIGHT_ENABLE()  cbi(PORTC, PC3)
#define LOCKED_ANTI()   cbi(PORTC, PC2)
#define SIGN_MAG()      sbi(PORTC, PC2)
#define RW_LCD()        sbi(PORTC, PC4) // Used to select switch input or
#define RD_BUTTONS()    cbi(PORTC, PC4) // LCD data buss

/* PORTD

    0 - in  Rx (no pullup)
    1 - out Tx (low)
    2 - in  RC0 (throttle)
    3 - in  RC1 (steering)
    4 - out PWM Right
    5 - out PWM Left
    6 - out LED2 (default high)
    7 - out Left Disable, default high
*/
#define DDRD_INIT 0xF2
#define PORTD_INIT 0xCD

#define LED2_ON() cbi(PORTD, PD6)
#define LED2_OFF() sbi(PORTD, PD6)
#define LEFT_DISABLE() sbi(PORTD, PD7)
#define LEFT_ENABLE() cbi(PORTD, PD7)

